@@ -12,8 +12,7 @@ static s6_servicedir_desc const s6_servicedir_file_list_[] =
   { .name = "timeout-finish", .type = S6_FILETYPE_UINT, .options = 0 },
   { .name = "max-death-tally", .type = S6_FILETYPE_UINT, .options = 0 },
   { .name = "down-signal", .type = S6_FILETYPE_NORMAL, .options = 0 },
-  { .name = "instance", .type = S6_FILETYPE_DIR, .options = 0 },
-  { .name = "instances", .type = S6_FILETYPE_DIR, .options = 0 },
+  { .name = "template", .type = S6_FILETYPE_DIR, .options = 0 },
   { .name = "data", .type = S6_FILETYPE_DIR, .options = 0 },
   { .name = "env", .type = S6_FILETYPE_DIR, .options = 0 },
   { .name = 0, .options = 0 }
