@@ -2,7 +2,7 @@
 
 #include <string.h>
 #include <skalibs/djbunix.h>
-#include <s6/s6-supervise.h>
+#include <s6/supervise.h>
 
 int s6_svstatus_write (char const *dir, s6_svstatus_t const *status)
 {
